import {
  Button,
  Card, Col, Grid, Icon,
  List,
  Modal,
  ModalService,
  Row, Text,
} from '@zeedhi/common';
import { ZdECharts } from '@zeedhi/zd-echarts';
import {
  Http,
  IDictionary,
  IEventParam,
  Metadata,
} from '@zeedhi/core';
import { DashboardGrid } from '@zeedhi/zd-dashboard-grid-common';
import { Kanban, KanbanColumn } from '@zeedhi/zd-kanban';

export class CrmNovoDashboardController {
  private dashModal!: Modal;

  public async openModal(path: string) {
    try {
      this.dashModal = await ModalService.createFromJson(path, true);
      this.dashModal.show();
    } catch (error) {
      console.error('Erro ao carregar:', error);
    }
  }

  public async mountCharts({ component }: any, route: string) {
    const newOptions = { ...component.options };
    await Http.get(route).then((response: IDictionary) => {
      if (response.status === 200) {
        response.data.data.forEach((dados: IDictionary) => {
          newOptions.series[0].data.push({
            name: dados.DESCRESULEVOL,
            value: dados.QUANTIDADE,
          });
        });
      }
    });
    component.setOption(newOptions);
  }

  public amountOfLosts() {
    const chart = Metadata.getInstance<ZdECharts>('loses-chart');
    const route = '/getLostsChart';
    this.mountCharts({ component: chart }, route);
  }

  public async getTasks({ component }: { component: Card }) {
    const rowAtrasadas = component.getChildInstance<Row>('row_outdated');
    const atrasadas = rowAtrasadas.getChildInstance<Text>('tasks_late');

    const rowHoje = component.getChildInstance<Row>('row_today');
    const hoje = rowHoje.getChildInstance<Text>('tasks_today');

    const rowFuturas = component.getChildInstance<Row>('row_future');
    const futuras = rowFuturas.getChildInstance<Text>('tasks_future');

    await Http.get('/getTasksIndex').then((response: IDictionary) => {
      if (response.status === 200) {
        atrasadas.text = response.data.data[0].QTDE_ATRASADAS.toString();
        hoje.text = response.data.data[0].QTDE_HOJE.toString();
        futuras.text = response.data.data[0].QTDE_FUTURAS.toString();
      }
    });
  }

  public async createTasksCards({ component }: { component: Row }) {
    const period = this.verifyPeriod(component);
    await Http.get(period).then((response: IDictionary) => {
      if (response.status === 200) {
        if (response.data && response.data.data && response.data.data.length) {
          response.data.data.forEach((element: any, pos: number) => {
            const headerTitle = this.createText('cardTitle', 'tasks_title', 'Tarefa');
            const nameHeader = this.createText('taskName', 'tasks_name', this.truncateString(element.DSFILACONTATO, 20));
            const rowHeader = this.createRow([headerTitle, nameHeader], 'card_header');

            const clienteName = this.createText('taskName', 'tasks_client', this.truncateString(element.NMPRINCIPALPARC, 20));
            const rowContent = this.createRow([clienteName], 'card_content');

            const subject = this.createText('taskAss', 'tasks_ass', 'Ass.: ');
            const rowFooter = this.createRow([subject], 'card_footer');

            const card = this.createCard([rowHeader, rowContent, rowFooter], { name: `card_${pos}` });
            component.addChild(card);
          });
        } else {
          const textNoTasks = this.createText('message', 'task_message', 'Não há tarefas agendadas, lembre-se de sempre monitorar suas atividades');
          const body = this.createRow([textNoTasks], 'card_content');
          component.addChild(body);
        }
      } else {
        const textNoTasks = this.createText('message', 'task_message', 'Não há tarefas agendadas, lembre-se de sempre monitorar suas atividades');
        const body = this.createRow([textNoTasks], 'card_content', { name: 'no_tasks' });
        component.addChild(body);
      }
      const textFooter = this.createText('footer', 'task_footer_text', 'Ver todas as tarefas');
      const footer = this.createRow([textFooter], 'task_footer', { name: 'row_task_footer' });
      component.addChild(footer);
    });
  }

  public createCard(childrens: any, optional: any = {}) {
    return {
      name: 'card',
      allowDuplicate: true,
      component: 'ZdCard',
      elevation: 0,
      cols: 3,
      cssClass: 'tasks_cards',
      children: childrens,
      ...optional,
    };
  }

  public createRow(childrens: any, styleClass: string = '', optional: any = {}) {
    return {
      name: 'card_row',
      allowDuplicate: true,
      component: 'ZdRow',
      cssClass: styleClass,
      children: childrens,
      ...optional,
    };
  }

  public createText(nome: string, styleClass: string, content: string) {
    return {
      name: nome,
      component: 'ZdText',
      allowDuplicate: true,
      text: content,
      cssClass: styleClass,
    };
  }

  public truncateString(str: string, maxLength: number) {
    if (str.length > maxLength) {
      return str.substring(0, maxLength).concat('...');
    }
    return str;
  }

  public verifyPeriod(component: any) {
    switch (component.parent.name) {
      case 'modal-tarefas-hoje':
        return '/getTasksToday';
        break;

      case 'modal-tarefas-atrasadas':
        return '/getTasksOutdated';
        break;

      case 'modal-tarefas-futuras':
        return '/getTasksFuture';
        break;

      default:
        return '/getTasksFuture';
        break;
    }
  }

  public async getOportunities({ component }: { component: Card }) {
    const rowOportunity = component.getChildInstance<Row>('row_oportunity');
    const oportunidades = rowOportunity.getChildInstance<Text>('oportunity_value');

    const rowPeriod = rowOportunity.getChildInstance<Row>('row_period').getChildInstance<Col>('col_period');
    const beginPeriod = rowPeriod.getChildInstance<Text>('oportunity_beginPeriod');
    const endPeriod = rowPeriod.getChildInstance<Text>('oportunity_endPeriod');

    const rowRate = rowOportunity.getChildInstance<Row>('row_period').getChildInstance<Col>('col_rate');
    const rate = rowRate.getChildInstance<Text>('oportunity_rate_value');
    const rateIcon = rowRate.getChildInstance<Icon>('oportunity_rate_icon');

    await Http.get('/getOportunitiesIndex').then((response: IDictionary) => {
      if (response.status === 200) {
        oportunidades.text = response.data.data[0].QTDE_CORRENTE.toString();
        beginPeriod.text = response.data.data[0].MES_ANTERIOR.substring(0, 3);
        endPeriod.text = response.data.data[0].MES_ATUAL.substring(0, 3);

        const qtdeCorrente = response.data.data[0].QTDE_CORRENTE;
        const qtdeAnterior = response.data.data[0].QTDE_ANTERIOR;

        const percentual = qtdeAnterior !== 0 ? ((qtdeCorrente - qtdeAnterior) / qtdeAnterior) * 100 : 0;
        const resultado = `${percentual.toFixed(2)}%`;

        rate.text = resultado;

        if (percentual > 0) {
          rowRate.cssClass += ' rateUpBox';
          rate.cssClass += ' rateUpText';
          rateIcon.cssClass += ' rateUpText';
          rateIcon.iconName = 'mdi-chevron-double-up';
        } else if (percentual < 0) {
          rowRate.cssClass += ' rateDownBox';
          rate.cssClass += ' rateDownText';
          rateIcon.cssClass += ' rateDownText';
          rateIcon.iconName = 'mdi-chevron-double-down';
        }
      }
    });
  }

  public async getAgenda({ component }: { component: Card }) {
    const rowCalls = component.getChildInstance<Row>('row_calls');
    const calls = rowCalls.getChildInstance<Text>('calls_value');

    const rowMeets = component.getChildInstance<Row>('row_meeting');
    const meets = rowMeets.getChildInstance<Text>('meet_value');

    await Http.get('/getAgendaIndex').then((response: IDictionary) => {
      if (response.status === 200) {
        calls.text = response.data.ligacoes;
        meets.text = response.data.reunioes;
      }
    });
  }

  public async openTodayModal(): Promise<void> {
    this.openModal('/crm_novo_dashboard/modal/tarefas_hoje');
  }

  public async openFutureModal(): Promise<void> {
    this.openModal('/crm_novo_dashboard/modal/tarefas_futuras');
  }

  public async openOutdatedModal(): Promise<void> {
    this.openModal('/crm_novo_dashboard/modal/tarefas_atrasadas');
  }

  public closeModal(): void {
    this.dashModal.destroy();
  }

  public getCurrentDate({ component }: { component: Text }) {
    const meses = ['jan', 'fev', 'mar', 'abr', 'mai', 'jun', 'jul', 'ago', 'set', 'out', 'nov', 'dez'];
    const dataAtual = new Date();
    const dia = dataAtual.getDate();
    const mes = meses[dataAtual.getMonth()];

    component.text = `${dia}/${mes}`;
  }

  public saveDash() {
    const dashboard = Metadata.getInstance<DashboardGrid>('crmDashboard');
    dashboard.saveChanges();
    dashboard.editingMode = false;
    dashboard.showGridLines = false;
  }

  public addCard() {
    const dashboard = Metadata.getInstance<DashboardGrid>('crmDashboard');
    dashboard.openModalToggleVisibilityCard();
  }

  public enterEditMode() {
    const dashboard = Metadata.getInstance<DashboardGrid>('crmDashboard');
    dashboard.editingMode = true;
    dashboard.showGridLines = true;
    dashboard.autoSave = true;
  }

  public getPageName() {
    return document.title.split(' | ')[0];
  }

  public createItemActions(actionName: string, iconName: string, optional: any = {}) {
    return {
      name: actionName,
      allowDuplicate: true,
      component: 'ZdButton',
      iconName,
      color: '#4F4F4F',
      small: true,
      icon: true,
      flat: true,
      ...optional,
    };
  }

  public createFavItem(nr: string | number, content: string, optional: any = {}) {
    return {
      name: `favCard_${nr}`,
      component: 'ZdListItem',
      allowDuplicate: true,
      title: content,
      cssClass: 'fav_item',
      ...optional,
    };
  }

  public getUsrFavorites({ component }: IEventParam<List>) {
    const usrFavorites = localStorage.getItem('LOGIN_FAVORITES');

    if (usrFavorites) {
      const fovorites = JSON.parse(usrFavorites);

      const events = {
        click: '{{CrmNovoDashboardController.removeFav}}',
      };

      component.items = [];

      fovorites.forEach((element: any) => {
        const actions = {
          leftField: this.createItemActions(`button_save-${element.nr}`, 'mdi-bookmark-outline', { events }),
          rightField: this.createItemActions(`button_remove-${element.nr}`, 'fa-regular fa-circle-xmark', { events }),
        };

        const favItem = this.createFavItem(element.nr, element.nm, actions);
        component.items.push(favItem);
      });
    }

    if (component.items.length < 6) {
      const optional = {
        leftField: this.createItemActions('button_create', 'mdi-plus-circle-outline'),
        events: {
          click: '{{CrmNovoDashboardController.addFavList}}',
        },
        cssClass: 'fav_item_add',
      };

      const favItem = this.createFavItem('add', 'Novo Atalho', optional);
      component.items.push(favItem);
    }
  }

  public async addFavList() {
    this.saveFavorites();

    this.openModal('/crm_novo_dashboard/modal/novo_atalho');
  }

  public saveFavorites() {
    const favoritesString = localStorage.getItem('LOGIN_FAVORITES');
    let favorites = [];

    if (favoritesString) {
      const parsedFavorites = JSON.parse(favoritesString);
      favorites = Array.isArray(parsedFavorites) ? parsedFavorites : [];
    }

    const lead = [{
      nr: `0${favorites.length + 1}`,
      nm: `nmparceiro 0${favorites.length + 1}`,
    }];

    favorites.push(...lead);
    localStorage.setItem('LOGIN_FAVORITES', JSON.stringify(favorites));

    this.reloadFavList();
  }

  public removeFav({ component }: IEventParam<Button>) {
    let idItem: any = component.name.split('-');
    idItem = idItem ? idItem[1] : null;

    const favoritesString = localStorage.getItem('LOGIN_FAVORITES');
    const parsedFavorites = favoritesString ? JSON.parse(favoritesString) : [];

    const index = parsedFavorites.findIndex((favorite: any) => favorite.nr === idItem);

    if (index >= 0) {
      parsedFavorites.splice(index, 1);
    }

    localStorage.setItem('LOGIN_FAVORITES', JSON.stringify(parsedFavorites));
    this.reloadFavList();
  }

  public reloadFavList() {
    const listFav: IEventParam<List> = { component: Metadata.getInstance<List>('FavList') };
    this.getUsrFavorites(listFav);
  }

  public async rowClick({ component }: any) {
    const dados: any = component.datasource.currentRow;
    await Http.get('/getLeadProgress', { params: dados }).then((response: IDictionary) => {
      if (response.status === 200) {
        const info: any = {
          nome: dados.NMCANDIDATO,
          nrParc: dados.NRPARCNEGOCIO,
          nrCand: dados.NRCANDIDATO,
          qtdTarefas: response.data.data[0].QTD_TAREFAS,
          ultiAtu: response.data.data[0].DTMOVIMENTACAO,
          vencimento: response.data.data[0].VENCIMENTO,
        };

        this.progressoModal(info);
      }
    });
  }

  public progressoModal(info: any) {
    this.dashModal = ModalService.create({
      name: 'modal-progresso',
      persistent: true,
      grid: {
        cols: 12,
      },
      children: [
        {
          name: 'header-modal-progresso',
          component: 'ZdHeader',
          color: 'transparent',
          cssClass: 'zd-pa-0 modalProgressHeader',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'title-modal-progresso',
              component: 'ZdText',
              text: 'Progresso',
              cssClass: 'title-modal-name',
            },
            {
              name: 'name-modal-progresso',
              component: 'ZdText',
              text: info.nome,
              cssClass: 'title-modal-cand',
            },
          ],
          rightSlot: [
            {
              name: 'close-modal-progresso',
              component: 'ZdButton',
              icon: true,
              iconName: 'mdi-close',
              events: {
                click: '{{CrmNovoDashboardController.closeModal}}',
              },
            },
          ],
        },
        {
          name: 'modal-progresso-dados',
          component: 'ZdRow',
          color: 'transparent',
          cssClass: 'progresso_buttons',
          padless: true,
          elevation: 0,
          children: [
            {
              name: 'modal-progresso-button-tarefas',
              component: 'ZdButton',
              label: `${info.qtdTarefas} Tarefas`,
              cssClass: 'zd-my-2',
              outline: true,
              iconName: 'mdi-briefcase-variant-outline',
            },
            {
              name: 'vencimento',
              component: 'ZdButton',
              label: `Vencimento em ${info.vencimento} dias`,
              cssClass: 'zd-my-2',
              outline: true,
              iconName: 'mdi-calendar-remove-outline',
            },
            {
              name: 'parado',
              component: 'ZdButton',
              label: `Parado há ${info.ultiAtu} dias`,
              cssClass: 'zd-my-2',
              outline: true,
              iconName: 'mdi-calendar-blank-outline',
            },
          ],
        },
        {
          name: 'modal-progresso-sdr',
          component: 'ZdRow',
          color: 'transparent',
          cssClass: 'progresso_button_sdr',
          padless: true,
          elevation: 0,
          children: [
            {
              name: 'registros',
              component: 'ZdButton',
              label: 'Registro de atividades SDR',
              cssClass: 'zd-my-2',
              outline: true,
              iconName: 'mdi-format-list-bulleted',
            },
          ],
        },
        {
          name: 'modal-progresso-kanban',
          component: 'ZdRow',
          color: 'transparent',
          cssClass: 'zd-pa-0',
          padless: true,
          elevation: 0,
          children: [
            {
              name: 'kanban-progresso',
              component: 'ZdKanban',
              columnKey: 'column',
              orderKey: 'order',
              draggable: false,
              height: 'auto',
              taskMetadata: {
                name: '<<row.name>>',
                component: 'ZdKanbanTask',
                elevation: 0,
                events: {
                  onMounted: '{{CrmNovoDashboardController.validaProximo}}',
                },
                children: [
                  {
                    name: '<<row.name>>',
                    component: 'ZdText',
                    text: '<<row.text>>',
                  },
                ],
              },
              columns: [
                {
                  id: '1',
                  name: 'concluida',
                  title: 'Etapa Concluídas',
                },
                {
                  id: '2',
                  name: 'atual',
                  title: 'Etapa Atual',
                  tasks: [],
                },
                {
                  id: '3',
                  name: 'proxima',
                  title: 'Próxima',
                  tasks: [],
                },
              ],
              datasource: {
                uniqueKey: 'id',
                route: `/getLeadProgressHist?NRPARCNEGOCIO=${info.nrParc}`,
                type: 'rest',
                limit: 5,
                lazyLoad: false,
                localSearch: false,
              },
            },
          ],
        },
        {
          name: 'modal-progresso-action',
          component: 'ZdRow',
          color: 'transparent',
          cssClass: 'progresso_act',
          padless: true,
          elevation: 0,
          children: [
            {
              name: 'act_01',
              component: 'ZdButton',
              label: 'Não',
              cssClass: 'modal_progress_act01',
              outline: false,
              iconName: 'mdi-thumb-down-outline',
              events: {
                click: '{{CrmNovoDashboardController.closeModal}}',
              },
            },
            {
              name: `act02_${info.nrParc}`,
              component: 'ZdButton',
              label: 'Sim, avançou',
              cssStyle: {
                color: ' #ffffff !important',
                'background-color': '#888888  !important',
              },
              outline: false,
              iconName: 'mdi-thumb-up-outline',
              events: {
                click: '{{CrmNovoDashboardController.updateProgess}}',
              },
            },
          ],
        },
      ],
    });

    this.dashModal.show();
  }

  public updateProgess({ component }: any) {
    const nrParc = component.name.split('_')[1];
    Http.post('/updateProgess', { NRPARCNEGOCIO: nrParc }).then((response: IDictionary) => {
      if (response.status === 200) {
        const kanban = Metadata.getInstance<Kanban>('kanban-progresso');
        kanban.datasource.clearFilter();

        const grid = Metadata.getInstance<Grid>('grid-progress');
        grid.datasource.clearFilter();
      }
    });
  }

  public validaProximo() {
    const tasks = Metadata.getInstance<KanbanColumn>('kanban-progresso-3');
    if (!(tasks.datasource.data.length)) {
      const row = Metadata.getInstance<Row>('modal-progresso-action');
      row.children = [row.children[0]];
    }
  }
}
